from __future__ import absolute_import, print_function

from invenio_pidstore.fetchers import FetchedPID
from oarepo_communities.converters import CommunityPIDValue
from oarepo_communities.proxies import current_oarepo_communities

from .providers import NREventsIdProvider


def nr_events_id_fetcher(record_uuid, data):
    """Fetch a record's identifiers.

    :param record_uuid: The record UUID.
    :param data: The record metadata.
    :returns: A :data:`invenio_pidstore.fetchers.FetchedPID` instance.
    """
    id_field = "control_number"
    return FetchedPID(  # FetchedPID je obyčejný namedtuple
        provider=NREventsIdProvider,
        pid_type=NREventsIdProvider.pid_type,
        pid_value=CommunityPIDValue(
            str(data[id_field]),
            current_oarepo_communities.get_primary_community_field(data))
    )



import os

EVENTS_ALLOWED_SCHEMAS = ['nr_events/nr-events-v1.0.0.json']
EVENTS_PREFERRED_SCHEMA = 'nr_events/nr-events-v1.0.0.json'

DRAFT_EVENT_PID_TYPE = 'dnrevt'
DRAFT_EVENT_RECORD = 'nr_events.record:DraftEventRecord'

PUBLISHED_EVENT_PID_TYPE = 'nrevt'
PUBLISHED_EVENT_RECORD = 'nr_events.record:PublishedEventRecord'

ALL_EVENTS_PID_TYPE = 'anrevt'
ALL_EVENTS_RECORD_CLASS = 'nr_events.record:AllEventRecord'

published_index_name = 'nr_events-nr-events-v1.0.0'
draft_index_name = 'draft-nr_events-nr-events-v1.0.0'
all_events_index_name = 'nr-all-events'

prefixed_published_index_name = os.environ.get('INVENIO_SEARCH_INDEX_PREFIX',
                                               '') + published_index_name
prefixed_draft_index_name = os.environ.get('INVENIO_SEARCH_INDEX_PREFIX', '') + draft_index_name
prefixed_all_events_index_name = os.environ.get('INVENIO_SEARCH_INDEX_PREFIX', '') + all_events_index_name



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 CIS UCT Prague.
#
# CIS events repository is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Default configuration."""

from __future__ import absolute_import, print_function

from invenio_records_rest.utils import allow_all, deny_all
from nr_common.search import community_search_factory
from oarepo_communities.links import community_record_links_factory

from nr_events.constants import PUBLISHED_EVENT_PID_TYPE, PUBLISHED_EVENT_RECORD, published_index_name, \
    DRAFT_EVENT_PID_TYPE, DRAFT_EVENT_RECORD, ALL_EVENTS_RECORD_CLASS, ALL_EVENTS_PID_TYPE, all_events_index_name
from nr_events.record import draft_index_name
from nr_events.search import GenericRecordsSearch

RECORDS_DRAFT_ENDPOINTS = {
    'events-community': {
        'draft': 'draft-events-community',
        'pid_type': PUBLISHED_EVENT_PID_TYPE,
        'pid_minter': 'nr_events',
        'pid_fetcher': 'nr_events',
        'default_endpoint_prefix': True,
        'max_result_window': 500000,
        'record_class': PUBLISHED_EVENT_RECORD,
        'search_index': published_index_name,
        'search_factory_imp': community_search_factory,

        'list_route': '/<community_id>/events/',
        'item_route': f'/<commpid(nrthe,model="events",record_class="nr_events.record:PublishedEventRecord"):pid_value>',

        'publish_permission_factory_imp': 'nr_common.permissions.publish_draft_object_permission_impl',
        'unpublish_permission_factory_imp': 'nr_common.permissions.unpublish_draft_object_permission_impl',
        'edit_permission_factory_imp': 'nr_common.permissions.update_object_permission_impl',
        'list_permission_factory_imp': allow_all,
        'read_permission_factory_imp': allow_all,
        'create_permission_factory_imp': deny_all,
        'update_permission_factory_imp': deny_all,
        'delete_permission_factory_imp': deny_all,
        'default_media_type': 'application/json',
        'links_factory_imp': community_record_links_factory,
        'search_class': GenericRecordsSearch,
        # 'indexer_class': CommitingRecordIndexer,
        'files': dict(
            # Who can upload attachments to a draft dataset record
            put_file_factory=deny_all,
            # Who can download attachments from a draft dataset record
            get_file_factory=allow_all,
            # Who can delete attachments from a draft dataset record
            delete_file_factory=deny_all
        )

    },
    'draft-events-community': {
        'pid_type': DRAFT_EVENT_PID_TYPE,
        'record_class': DRAFT_EVENT_RECORD,

        'list_route': '/<community_id>/events/draft/',
        'item_route': f'/<commpid(nrthe,model="events/draft",record_class="nr_events.record:DraftEventRecord"):pid_value>',
        'search_index': draft_index_name,
        'links_factory_imp': community_record_links_factory,
        'search_factory_imp': community_search_factory,
        'search_class': GenericRecordsSearch,
        'search_serializers': {
            'application/json': 'oarepo_validate:json_search',
        },
        'record_serializers': {
            'application/json': 'oarepo_validate:json_response',
        },

        'create_permission_factory_imp': 'nr_common.permissions.create_draft_object_permission_impl',
        'update_permission_factory_imp': 'nr_common.permissions.update_draft_object_permission_impl',
        'read_permission_factory_imp': 'nr_common.permissions.read_draft_object_permission_impl',
        'delete_permission_factory_imp': 'nr_common.permissions.delete_draft_object_permission_impl',
        'list_permission_factory_imp': 'nr_common.permissions.list_draft_object_permission_impl',
        'record_loaders': {
            'application/json': 'oarepo_validate.json_files_loader',
            'application/json-patch+json': 'oarepo_validate.json_loader'
        },
        'files': dict(
            put_file_factory='nr_common.permissions.put_draft_file_permission_impl',
            get_file_factory='nr_common.permissions.get_draft_file_permission_impl',
            delete_file_factory='nr_common.permissions.delete_draft_file_permission_impl'
        )

    },
    'events': {
        'draft': 'draft-events',
        'pid_type': PUBLISHED_EVENT_PID_TYPE + '-events',
        'pid_minter': 'nr_events',
        'pid_fetcher': 'nr_events',
        'default_endpoint_prefix': True,
        'max_result_window': 500000,
        'record_class': ALL_EVENTS_RECORD_CLASS,
        'search_index': published_index_name,

        'list_route': '/events/',
        'item_route': f'/not-really-used',
        'publish_permission_factory_imp': deny_all,
        'unpublish_permission_factory_imp': deny_all,
        'edit_permission_factory_imp': deny_all,
        'list_permission_factory_imp': allow_all,
        'read_permission_factory_imp': allow_all,
        'create_permission_factory_imp': deny_all,
        'update_permission_factory_imp': deny_all,
        'delete_permission_factory_imp': deny_all,
        'default_media_type': 'application/json',
        'links_factory_imp': community_record_links_factory,
        'search_class': GenericRecordsSearch,
        # 'indexer_class': CommitingRecordIndexer,
        'files': dict(
            # Who can upload attachments to a draft dataset record
            put_file_factory=deny_all,
            # Who can download attachments from a draft dataset record
            get_file_factory=allow_all,
            # Who can delete attachments from a draft dataset record
            delete_file_factory=deny_all
        )
    },
    'draft-events': {
        'pid_type': DRAFT_EVENT_PID_TYPE + '-draft-events',
        'record_class': ALL_EVENTS_RECORD_CLASS,

        'list_route': '/events/draft/',
        'item_route': f'/not-really-used',
        'search_index': draft_index_name,
        'links_factory_imp': community_record_links_factory,
        'search_class': GenericRecordsSearch,
        'search_serializers': {
            'application/json': 'oarepo_validate:json_search',
        },
        'record_serializers': {
            'application/json': 'oarepo_validate:json_response',
        },

        'create_permission_factory_imp': deny_all,
        'update_permission_factory_imp': deny_all,
        'read_permission_factory_imp': 'nr_common.permissions.read_draft_object_permission_impl',
        'delete_permission_factory_imp': deny_all,
        'list_permission_factory_imp': 'nr_common.permissions.list_draft_object_permission_impl',
        'files': dict(
            put_file_factory=deny_all,
            get_file_factory='nr_common.permissions.get_draft_file_permission_impl',
            delete_file_factory=deny_all
        )
    }
}


RECORDS_REST_ENDPOINTS = {
    'all-events': dict(
        pid_type=ALL_EVENTS_PID_TYPE,
        pid_minter='nr_all',
        pid_fetcher='nr_all',
        default_endpoint_prefix=True,
        record_class=ALL_EVENTS_RECORD_CLASS,
        search_class=GenericRecordsSearch,
        search_index=all_events_index_name,
        search_serializers={
            'application/json': 'oarepo_validate:json_search',
        },
        list_route='/events/all/',
        links_factory_imp=community_record_links_factory,
        default_media_type='application/json',
        max_result_window=10000,
        # not used really
        item_route=f'/events/'
                   f'/not-used-but-must-be-present',
        list_permission_factory_imp='nr_common.permissions.list_all_object_permission_impl',
        create_permission_factory_imp=deny_all,
        delete_permission_factory_imp=deny_all,
        update_permission_factory_imp=deny_all,
        read_permission_factory_imp=deny_all,
        record_serializers={
            'application/json': 'oarepo_validate:json_response',
        },
        use_options_view=False
    ),
    'community-events': dict(
        pid_type=ALL_EVENTS_PID_TYPE + '-community-all',
        pid_minter='nr_all',
        pid_fetcher='nr_all',
        default_endpoint_prefix=True,
        record_class=ALL_EVENTS_RECORD_CLASS,
        search_class=GenericRecordsSearch,
        search_index=all_events_index_name,
        search_factory_imp=community_search_factory,
        search_serializers={
            'application/json': 'oarepo_validate:json_search',
        },
        list_route='/<community_id>/events/all/',
        links_factory_imp=community_record_links_factory,
        default_media_type='application/json',
        max_result_window=10000,
        # not used really
        item_route=f'/event/'
                   f'/not-used-but-must-be-present',
        list_permission_factory_imp='nr_common.permissions.list_all_object_permission_impl',
        create_permission_factory_imp=deny_all,
        delete_permission_factory_imp=deny_all,
        update_permission_factory_imp=deny_all,
        read_permission_factory_imp=deny_all,
        record_serializers={
            'application/json': 'oarepo_validate:json_response',
        },
        use_options_view=False
    )
}


FILTERS = {
}

POST_FILTERS = {
}

RECORDS_REST_FACETS = {
}

RECORDS_REST_SORT_OPTIONS = {
}

RECORDS_REST_DEFAULT_SORT = {
}

"""Set default sorting options."""



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Thesis ID provider."""

from __future__ import absolute_import, print_function

from invenio_pidstore.models import PIDStatus
from invenio_pidstore.providers.base import BaseProvider

from nr_common.models import NRIdentifier
from nr_common.providers import NRIdProvider


class NREventsIdProvider(NRIdProvider):
    pid_type = 'nrevt'



import os

from flask import url_for
from invenio_records.api import Record
from oarepo_communities.converters import CommunityPIDValue
from oarepo_communities.proxies import current_oarepo_communities
from oarepo_communities.record import CommunityRecordMixin
from oarepo_records_draft.record import InvalidRecordAllowedMixin, DraftRecordMixin
from oarepo_references.mixins import ReferenceEnabledRecordMixin
from oarepo_validate import SchemaKeepingRecordMixin, MarshmallowValidatedRecordMixin

from .constants import EVENTS_ALLOWED_SCHEMAS, EVENTS_PREFERRED_SCHEMA, published_index_name, draft_index_name, \
    all_events_index_name
from .marshmallow import EventsMetadataSchemaV1


class EventBaseRecord(SchemaKeepingRecordMixin,
                      MarshmallowValidatedRecordMixin,
                      ReferenceEnabledRecordMixin,
                      CommunityRecordMixin,
                      Record,
                      ):
    ALLOWED_SCHEMAS = EVENTS_ALLOWED_SCHEMAS
    PREFERRED_SCHEMA = EVENTS_PREFERRED_SCHEMA
    MARSHMALLOW_SCHEMA = EventsMetadataSchemaV1


class PublishedEventRecord(InvalidRecordAllowedMixin, EventBaseRecord):
    index_name = published_index_name

    @property
    def canonical_url(self):
        return url_for('invenio_records_rest.events-community_item',
                       pid_value=CommunityPIDValue(
                           self['control_number'],
                           current_oarepo_communities.get_primary_community_field(self)),
                       _external=True)


class DraftEventRecord(DraftRecordMixin, EventBaseRecord):
    index_name = draft_index_name

    @property
    def canonical_url(self):
        return url_for('invenio_records_rest.draft-events-community_item',
                       pid_value=CommunityPIDValue(
                           self['control_number'],
                           current_oarepo_communities.get_primary_community_field(self)),
                       _external=True)


class AllEventRecord(SchemaKeepingRecordMixin, CommunityRecordMixin, Record):
    ALLOWED_SCHEMAS = EVENTS_ALLOWED_SCHEMAS
    PREFERRED_SCHEMA = EVENTS_PREFERRED_SCHEMA
    index_name = all_events_index_name

    @property
    def canonical_url(self):
        if not self.get('oarepo:draft'):
            endpoint = 'invenio_records_rest.events-community_item'
        else:
            endpoint = 'invenio_records_rest.draft-events-community_item'

        return url_for(endpoint,
                       pid_value=CommunityPIDValue(
                           self['control_number'],
                           current_oarepo_communities.get_primary_community_field(self)),
                       _external=True)
